<?php


class wps_ic_backup extends wps_ic {
	
	public $backup_log;
	public $local_backup_path;
	
	
	public function __construct() {
		global $wpdb;
		$this->backup_log        = $wpdb->prefix . 'ic_backups';
		$this->local_backup_path = ABSPATH . 'wp-content/wps_ic_backup/';
		$this->local_backup_uri  = site_url('wp-content/wps_ic_backup/');
		$backup_log              = get_option($this->backup_log);
	}
	
	
	/**
	 * Localy backup file
	 *
	 * @param $attachment_id
	 */
	public function local_backup($attachment_id) {
		global $wpdb,$wps_ic;
		
		$this->change_limits();
		
		$file_data = get_attached_file($attachment_id);

    // Write Log
    $wps_ic->log->write_log($attachment_id, 'Started backup!', '1', 'compress');

		if (!empty($file_data)) {

      // Write Log
      $wps_ic->log->write_log($attachment_id, 'Started backup not empty file data!', '1', 'compress');

			// Setup backup filename
			$backup_filename = basename($file_data);

			if (file_exists($this->local_backup_path . $backup_filename)) {
			  return false;
      }

      // Copy the actual file
      #$this->copy_file($file_data, $this->local_backup_path . $backup_filename);


			// Write backup log
			$query = $wpdb->prepare("INSERT INTO " . $this->backup_log . " (attachment_ID, path, url, filename, size, created) VALUES (%s, %s, %s, %s, %s, %s) ON DUPLICATE KEY UPDATE path=%s, url=%s, filename=%s, size=%s, created=%s",
			                        $attachment_id,
			                        $this->local_backup_path . $backup_filename,
			                        $this->local_backup_uri . $backup_filename,
			                        $backup_filename,
			                        filesize($file_data),
			                        current_time('mysql'),
				/* update */
				                    $this->local_backup_path . $backup_filename,
				                    $this->local_backup_uri . $backup_filename,
				                    $backup_filename,
				                    filesize($file_data),
				                    current_time('mysql'));

			$query = $wpdb->query($query);

			return $query;
		}
	}
	
	
	public function change_limits() {
	  if (!defined('WP_MEMORY_LIMIT')) {
      define('WP_MEMORY_LIMIT', '512M');
    }
		ini_set('memory_limit', '512M');
		ini_set('max_execution_time', '300');
	}
	
	
	/**
	 * Copy File
	 *
	 * @param $source
	 * @param $destination
	 */
	public function copy_file($source, $destination) {
		if (file_exists($source)) {
			copy($source, $destination);
		}
	}
	
	
	/**
	 * Restore local file with att ID
	 *
	 * @param $attachment_id
	 */
	public function local_restore($attachment_id) {
		global $wpdb;
		
		$this->change_limits();
		
		// Fetch data for attached file
		$file_data = get_attached_file($attachment_id);
		
		// Does file and data really exist?
		if (!empty($file_data)) {
			
			// File path
			$file_name = basename($file_data);
			$file_path = str_replace($file_name, '', $file_data);
			
			// Fetch backup log
			$backup = $wpdb->prepare("SELECT * FROM " . $this->backup_log . " WHERE attachment_ID=%s ORDER BY ID DESC LIMIT 1", $attachment_id);
			$backup = $wpdb->get_results($backup);
			
			if (!empty($backup)) {
				// Fetch last backup
				$last_backup_filename = $backup[0]->filename;
				
				// Copy the actual file
				unlink($file_path . $file_name);
				$copy = $this->copy_file($this->local_backup_path . $last_backup_filename, $file_path . $file_name);
				
				// Generate attachment metadata
				$fullsizepath = get_attached_file($attachment_id);
				$attach_data  = wp_generate_attachment_metadata($attachment_id, $fullsizepath);
				
				// Update attachment metadata
				wp_update_attachment_metadata($attachment_id, $attach_data);
				
				// Delete compress data
				delete_post_meta($attachment_id, 'wps_ic_compress_data');
				
				// Response
				wp_send_json_success();
			}
		}
		
		// Error
		wp_send_json_error();
	}
	
}