<?php


/**
 * Class - Controller
 */
class wps_ic_controller extends wps_ic {

  public $options;


  public function __construct() {

    $this->options = new wps_ic_options();

    // Register new columns
    add_action('wps_ic_pro_features', array(__CLASS__, 'pro_features'), 10);

    // Upon attachment delete
    add_action('delete_attachment', array(__CLASS__, 'delete_attachment'));

    // check for activated conflict plugins
    add_action('admin_notices', array(__CLASS__, 'wps_display_admin_notices'));

    //deactivate plugin
    add_action('admin_post_wps_ic_deactivate_plugin', array(__CLASS__, 'wps_deactivate_plugin'));
  }


  public static function wps_deactivate_plugin() {
    if (!wp_verify_nonce($_GET['_wpnonce'], 'wps_ic_deactivate_plugin_nonce')) {
      wp_nonce_ays('');
    }

    deactivate_plugins($_GET['plugin']);
    wp_safe_redirect(wp_get_referer());
    die();
  }


  public static function wps_display_admin_notices() {
    $extraStyle = "style='border-left: 4px solid#ff0000;'";
    $plugins = self::get_conflicts();
    if (count($plugins)) {
      ?>
      <div class='notice notice-warning' id='wp-compress-notice-conflict' <?php echo($extraStyle); ?>>
        <h3>WP Compress - Conflicts found
        </h3>Using WP Compress while other image optimization plugins are active can lead to corrupt images or unpredictable results. We recommend to deactivate the following plugin(s):
        <?php
        echo('<ul>');
        foreach ($plugins as $plugin) {
          echo('<li class="wps-ic-conflict-plugin"><strong>' . $plugin['name'] . '</strong>');
          echo('<a href="' . wp_nonce_url(admin_url('admin-post.php?action=wps_ic_deactivate_plugin&plugin=' . urlencode($plugin['path'])), 'wps_ic_deactivate_plugin_nonce') . '" class="button">Deactivate</a>');
        }
        echo("</ul>");

        ?>
      </div>
      <?php
    }
  }


  /**
   * Find other similar plugins for notice
   * @return array
   */
  public static function get_conflicts() {

    $conflict_plugins = array(
      'WP Smush Pro - Image Optimization' => 'wp-smush-pro/wp-smush.php',
      'WP Smush - Image Optimization' => 'wp-smushit/wp-smush.php',
      'Imagify Image Optimizer' => 'imagify/imagify.php',
      'Compress JPEG & PNG images (TinyPNG)' => 'tiny-compress-images/tiny-compress-images.php',
      'Kraken.io Image Optimizer' => 'kraken-image-optimizer/kraken.php',
      'Optimus - WordPress Image Optimizer' => 'optimus/optimus.php',
      'EWWW Image Optimizer' => 'ewww-image-optimizer/ewww-image-optimizer.php',
      'ImageRecycle pdf & image compression' => 'imagerecycle-pdf-image-compression/wp-image-recycle.php',
      'CheetahO Image Optimizer' => 'cheetaho-image-optimizer/cheetaho.php',
      'Zara 4 Image Compression' => 'zara-4/zara-4.php',
      'Prizm Image' => 'prizm-image/wp-prizmimage.php',
      'CW Image Optimizer' => 'cw-image-optimizer/cw-image-optimizer.php',
      'ShortPixel' => 'shortpixel-image-optimiser/wp-shortpixel.php');

    $found = array();

    // Go through plugin lists
    foreach ($conflict_plugins as $name => $path) {
      if (is_plugin_active($path)) {
        $found[] = array('name' => $name, 'path' => $path);
      }
    }

    return $found;
  }


  /**
   * Delete attachment
   * @param $id
   */
  public static function delete_attachment($id) {
    global $wps_ic, $wpdb;

    $filename = $wpdb->get_var("SELECT filename FROM " . $wpdb->prefix . "ic_backups WHERE attachment_ID='" . $id . "'");

    if ($filename && $filename != '') {

      // Delete File
      $file_path = WP_CONTENT_DIR . '/wps_ic_backup/' . $filename;
      if (file_exists($file_path)) {
        unlink($file_path);
      }

      // Delete backup record
      $wpdb->query("DELETE FROM " . $wpdb->prefix . "ic_backups WHERE attachment_ID='" . $id . "'");

      // Remove from Compressed
      $wpdb->query("UPDATE " . $wpdb->prefix . "ic_compressed SET restored='1' WHERE attachment_ID='" . $id . "'");

      // Remove from Queue
      $wpdb->query("DELETE FROM " . $wpdb->prefix . "ic_queue WHERE attachment_ID='" . $id . "'");

      // Remove from Queue
      $bulk_compress_queue = get_option(WPS_IC_BULK_PROCESS);
      unset($bulk_compress_queue[ $id ]);
      update_option(WPS_IC_BULK_PROCESS, $bulk_compress_queue);
    }
  }


  public function pro_features() {
    if (!class_exists('wps_ic_pro')) {
      include_once WPS_IC_DIR . 'templates/pages/settings.php';
    } else {
      include_once WPS_IC_PRO_DIR . 'templates/settings.php';
    }
  }


  /**
   * Run the core
   */
  public function core() {
    // Is plugin active?
  }


}