<?php


/**
 * Class - Tests
 */
class wps_ic_tests extends wps_ic {


  public function __construct() {
    if (is_admin()) {
      // Check API status
      $this->add_ajax('ic_run_tests');
    }
  }


  public function add_ajax($hook) {
    add_action('wp_ajax_' . $hook, array($this, $hook));
  }


  public function run() {
    $tests = get_option('wps_ic_tests');

    echo '<a href="#" class="wps-ic-run-tests button button-primary">Run Tests</a>';
    echo '<div class="wps-ic-spinner-tests" style="display:none;"><img src="' . WPS_IC_URI . 'assets/images/spinner.svg" /></div>';

    if ( ! empty($tests)) {
      echo '&nbsp;<a href=#" class="button button-primary wps-ic-view-results">View Results</a>';
      echo '<div class="wps-ic-test-results" style="display:none;">';
      echo $tests;
      echo '</div>';
    } else {
      echo '<div class="wps-ic-test-results" style="display:none;"></div>';
    }

  }


  public function ic_run_tests() {
    $output = '';

    $output .= '<ul class="wps-ic-tests">';
    $output .= '<li><strong>Server Uptime:</strong> ' . $this->get_uptime() . '</li>';
    $output .= '<li><strong>Server Speed:</strong> ' . $this->get_speed() . '</li>';
    $output .= '<li><strong>Server Benchmark:</strong> ' . $this->get_benchmark() . '</li>';
    $output .= '<li><strong>PHP Version:</strong> ' . $this->get_phpversion() . '</li>';
    $output .= '<li><strong>WP Version:</strong> ' . $this->get_wpversion() . '</li>';
    $output .= '<li><strong>MySQL Connection:</strong> ' . $this->get_mysql() . '</li>';
    $output .= '<li><strong>File permissions:</strong> ' . $this->get_file_permissions() . '</li>';
    $output .= '<li><strong>API Communication:</strong> ' . $this->get_api() . '</li>';
    $output .= '</ul>';

    update_option('wps_ic_tests', $output);

    wp_send_json_success($output);
  }

  /**
   * Get server uptime
   * @return string
   */
  public function get_uptime() {
    $uptime = exec("uptime", $system);
    if ( ! empty($uptime)) {
      return $uptime;
    }

    return 'Not able to verify.';
  }

  /**
   * Call API speed
   * @return string
   */
  public function get_speed() {
    $url = 'http://devapi.wpcompress.com';

    $start = microtime(true);
    $call  = wp_remote_get($url);
    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $end  = microtime(true);
      $time = $end - $start;
    }

    return round($time, 2) . ' seconds';

  }


  /**
   * Get benchmark
   * @return int|string
   */
  public function get_benchmark() {
    $total     = 0;

    $test_ifElse = $this->test_IfElse();
    $total += $test_ifElse;

    $test_Loops = $this->test_Loops();
    $total += $test_Loops;

    $test_Math = $this->test_Math();
    $total += $test_Math;

    $test_StringManipulation = $this->test_StringManipulation();
    $total += $test_StringManipulation;


    return round($total, 2) . ' seconds';
  }

  /**
   * Get Installed PHP Version
   * @return string
   */
  public function get_phpversion() {
    $version = phpversion();

    if (version_compare($version, '7.0.0') >= 0) {
      return 'OK (' . $version . ')';
    } else {
      return 'Bad (You are running ' . $version . ', while we recommend at least 7.0.0)';
    }

  }

  /**
   * Get WordPress Core Version
   * @return string
   */
  public function get_wpversion() {
    $version = get_bloginfo('version');

    if ($version >= 4.8) {
      return 'OK (' . $version . ')';
    } else {
      return 'Bad (You are running ' . $version . ', while we recommend at least 4.8.0)';
    }

  }

  /**
   * Get Mysql
   * @return string
   */
  public function get_mysql() {
    global $wpdb;

    $query = $wpdb->query("SELECT * FROM " . $wpdb->prefix . "ic_compressed");
    if ($query) {
      return 'OK';
    } else {
      return 'We were not able to see our database table! (ic_compressed)';
    }
  }

  /**
   * Folder & File write privileges
   * @return string
   */
  public function get_file_permissions() {
    if ( ! file_exists(ABSPATH . 'wp-content/wps_ic_backup')) {
      return 'Backup folder does not exist, please create it on ' . ABSPATH . 'wp-content/wps_ic_backup/';
    }

    $fp = fopen(ABSPATH . 'wp-content/wps_ic_backup/test.txt', 'w+');
    if (fwrite($fp, 'test')) {
      return 'OK';
    } else {
      return 'Bad, we were not able to write to ' . ABSPATH . 'wp-content/wps_ic_backup/test.txt';
    }

  }

  public function get_api() {
    global $wps_ic;
    $api = 'http://devapi.wpcompress.com';

    // Get first image
    $attachments = get_posts(array('post_type' => 'attachment', 'posts_per_page' => '1'));
    if ($attachments) {
      foreach ($attachments as $attachment) {
        $attach_id = $attachment->ID;
      }
    }

    // Get all image sizes
    $thumbs         = array();
    $thumbs['full'] = wp_get_attachment_image_src($attach_id, 'full');
    $thumbs['full'] = $thumbs['full'][0];
    $thumbs         = json_encode($thumbs);

    $settings = $wps_ic->options->get_settings();

    $api = $api . '?bulk=true&images=' . $thumbs . '&q=' . $settings['optimization'] . '&resize_max=0&preserve_exif=0&apikey=' . $wps_ic::$api_key . '&force=true&attachment_ID=' . $attach_id;

    $call = wp_remote_get($api, array('timeout' => 120));
    $code = wp_remote_retrieve_response_code($call);

    if ($code == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);
      if ($body->success == true) {
        return 'OK';
      }
    }

    return 'We were not able to communicate with API.';
  }


  public function test_Math($count = 140000) {
    $time_start    = microtime(true);
    $mathFunctions = array("abs", "acos", "asin", "atan", "bindec", "floor", "exp", "sin", "tan", "pi", "is_finite", "is_nan", "sqrt");
    foreach ($mathFunctions as $key => $function) {
      if ( ! function_exists($function)) {
        unset($mathFunctions[ $key ]);
      }
    }
    for ($i = 0; $i < $count; $i ++) {
      foreach ($mathFunctions as $function) {
        $r = call_user_func_array($function, array($i));
      }
    }

    return number_format(microtime(true) - $time_start, 3);
  }


  public function test_StringManipulation($count = 130000) {
    $time_start      = microtime(true);
    $stringFunctions = array("addslashes", "chunk_split", "metaphone", "strip_tags", "md5", "sha1", "strtoupper", "strtolower", "strrev", "strlen", "soundex", "ord");
    foreach ($stringFunctions as $key => $function) {
      if ( ! function_exists($function)) {
        unset($stringFunctions[ $key ]);
      }
    }
    $string = "the quick brown fox jumps over the lazy dog";
    for ($i = 0; $i < $count; $i ++) {
      foreach ($stringFunctions as $function) {
        $r = call_user_func_array($function, array($string));
      }
    }

    return number_format(microtime(true) - $time_start, 3);
  }


  public function test_Loops($count = 19000000) {
    $time_start = microtime(true);
    for ($i = 0; $i < $count; ++ $i) {
      ;
    }
    $i = 0;
    while ($i < $count) {
      ++ $i;
    }

    return number_format(microtime(true) - $time_start, 3);
  }


  public function test_IfElse($count = 9000000) {
    $time_start = microtime(true);
    for ($i = 0; $i < $count; $i ++) {
      if ($i == - 1) {
      } elseif ($i == - 2) {
      } else if ($i == - 3) {
      }
    }

    return number_format(microtime(true) - $time_start, 3);
  }


}