<?php
/*
 * Plugin name: WP Compress - Image Optimizer
 * Plugin URI: https://www.wpcompress.com
 * Author: WP Compress
 * Author URI: https://www.wpcompress.com
 * Version: 2.0.0
 * Description: Compress and optimize images to shrink file size, improve load times, boost PageSpeed scores and supercharge your SEO rankings with WP Compress Image Optimization.
 */

include 'defines.php';
include 'addons/nightowl/night-owl.php';

spl_autoload_register(function($class_name) {
  $class_name = str_replace('wps_ic_', '', $class_name);
  $class_name = $class_name . '.class.php';
  if (file_exists(WPS_IC_DIR . 'classes/' . $class_name)) {
    include 'classes/' . $class_name;
  }
});

// Plugin Auto Update
require 'update/plugin-update-checker.php';
$myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
  'http://version.wpcompress.com/plugin.json',
  __FILE__,
  'wp_compress',
  0
);


class wps_ic {

  public static $slug;
  public static $version;
  public static $api_key;

  public $options;
  public $notices;
  public $enqueues;
  public $templates;
  public $menu;
  public $ajax;
  public $backup;
  public $media_library;
  public $compress;
  public $controller;
  public $log;
  public $nextgen;
  public $requests;
  public $bulk;
  public $queue;
  public $stats;
  public $tester;


  /**
   * Our main class constructor
   */
  public function __construct() {
    global $wps_ic;

    $this->options = new wps_ic_options();

    add_filter('cron_schedules', array('wps_ic', 'wp_ic_cron_timming'));

    // Basic plugin info
    self::$slug    = 'wps_ic';
    self::$version = '2.0.0';
    self::$api_key = $this->options->get_option('api_key');

    if ( ! defined('ALLOW_UNFILTERED_UPLOADS')) {
      define('ALLOW_UNFILTERED_UPLOADS', true);
    }

    $this->tester         = new wps_ic_tester();
    $this->cron           = new wps_ic_cron();
    $this->log            = new wps_ic_log();
    $this->nextgen        = new wps_ic_nextgen();
    $this->menu           = new wps_ic_menu();
    $this->enqueues       = new wps_ic_enqueues();
    $this->templates      = new wps_ic_templates();
    $this->notices        = new wps_ic_notices();
    $this->ajax           = new wps_ic_ajax();
    $this->demo_ajax      = new wps_ic_demo_ajax();
    $this->backup         = new wps_ic_backup();
    $this->media_library  = new wps_ic_media_library();
    $this->compress       = new wps_ic_compress();
    $this->controller     = new wps_ic_controller();
    $this->requests       = new wps_ic_requests();
    $this->bulk           = new wps_ic_bulk();
    $this->queue          = new wps_ic_queue();
    $this->stats          = new wps_ic_stats();
    $this->tests          = new wps_ic_tests();
    $this->remote_restore = new wps_ic_remote_restore();
    $this->upgrade        = new wps_ic_upgrade();
    $this->unlocks        = new wps_ic_unlocks();
    $this->popups        = new wps_ic_popups();

    // What is unlocked?
    $unlocks = get_option(WPS_IC_SETTINGS);

    if (!empty($unlocks) && !empty($unlocks['unlocks'])) {
      $unlocks = $unlocks['unlocks'];

      if (!empty($unlocks) && !empty($unlocks['night_owl'])) {
        $nightowl = new wps_no();
      }
    }

    // Change PHP Limits
    $this->backup->change_limits();

    $wps_ic = $this;
    if ( ! empty($_GET['debug'])) {
      do_action('wps_ic_cron_compress');
      die();
    }

  }


  public static function wp_ic_cron_timming($schedules) {
    // add a 'weekly' schedule to the existing set
    $schedules['two_minutes'] = array(
      'interval' => 60 * 2,
      'display'  => __('Every two minutes')
    );

    return $schedules;
  }


  /**
   * WP Init helper
   */
  public static function init() {
    $class = __CLASS__;
    new $class;
  }


  /**
   * Activation of the plugin
   */
  public static function activation() {

    $unlocks = get_option(WPS_IC_SETTINGS);
    $unlocks = $unlocks['unlocks'];

    // Setup Default Options
    $options = new wps_ic_options();
    $options->set_defaults();

    // Setup Database
    $database = new wps_ic_database();
    $database->create_tables();

    // Schedules
    add_filter('cron_schedules', array('wps_ic', 'wp_ic_cron_timming'));

    // Setup Stats
    update_option(WPS_IC_STATS_BYTES, 0);
    update_option(WPS_IC_STATS_FILES, 0);
    update_option(WPS_IC_STATS_AVG_REDUCTION, array());

    // Create backup path
    if ( ! file_exists(ABSPATH . 'wp-content/wps_ic_backup/')) {
      mkdir(ABSPATH . 'wp-content/wps_ic_backup/');
    }

    // Setup bulk cron
    if ( ! wp_next_scheduled('wps_ic_cron_compress')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_cron_compress');
    }

    // Setup thumbnail cron
    if ( ! wp_next_scheduled('wps_ic_compress')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_compress');
    }

    // Setup thumbnail cron
    if ( ! wp_next_scheduled('wps_ic_fetch_thumbnails')) {
      wp_schedule_event(time(), 'two_minutes', 'wps_ic_fetch_thumbnails');
    }

    if ( ! wp_next_scheduled('wps_ic_single_thumbnail')) {
      wp_schedule_event(time(), 'daily', 'wps_ic_single_thumbnail');
    }

    if (!empty($unlocks) && !empty($unlocks['night_owl'])) {

      if ( ! wp_next_scheduled('wps_no_worker')) {
        wp_schedule_event(time(), 'hourly', 'wps_no_worker');
      }

    }

  }


  /**
   * Deactivation of the plugin
   */
  public static function deactivation() {
    global $wpdb;

    // Setup Database
    delete_option(WPS_IC_CRON_QUEUE);
    delete_option(WPS_IC_BULK_PROCESS);
    delete_option(WPS_IC_BULK);
    delete_option(WPS_IC_SAVINGS);
    delete_option(WPS_IC_AVG_SAVINGS);

    wp_unschedule_event(time(), 'wps_ic_compress');
    wp_unschedule_event(time(), 'wps_ic_daily_thumbnail');
    wp_unschedule_event(time(), 'wps_ic_single_thumbnail');
    wp_unschedule_event(time(), 'wps_ic_fetch_thumbnails');
    wp_unschedule_event(time(), 'wps_no_worker');
  }


}


add_action('init', array('wps_ic', 'init'));
register_activation_hook(__FILE__, array('wps_ic', 'activation'));
register_deactivation_hook(__FILE__, array('wps_ic', 'deactivation'));